#include <gtk/gtklabel.h>
#include <gtk/gtkpixmap.h>
#include <gtk/gtktable.h>
#include "gtktreeitempix.h"


static void gtk_tree_item_pix_class_init (GtkTreeItemPixClass *klass);
static void gtk_tree_item_pix_init       (GtkTreeItemPix *tree_item_pix);
void tree_item_pix_collapse              (GtkWidget *item, gpointer data);
void tree_item_pix_expand                (GtkWidget *item, gpointer data);

GtkType
gtk_tree_item_pix_get_type (void)
{

  static GtkType tree_item_pix_type = 0;

  if (!tree_item_pix_type)
    {
      GtkTypeInfo tree_item_pix_info =
      {
        "GtkTreeItemPix",
        sizeof (GtkTreeItemPix),
        sizeof (GtkTreeItemPixClass),
        (GtkClassInitFunc) gtk_tree_item_pix_class_init,
        (GtkObjectInitFunc) gtk_tree_item_pix_init,
        /* reserved_1 */ NULL,
        /* reserved_2 */ NULL,
        (GtkClassInitFunc) NULL,
      };

      tree_item_pix_type = gtk_type_unique (gtk_tree_item_get_type (),
                                            &tree_item_pix_info);
    }

  return tree_item_pix_type;
}

static void
gtk_tree_item_pix_class_init (GtkTreeItemPixClass *class)
{
}

static void
gtk_tree_item_pix_init (GtkTreeItemPix *tree_item_pix)
{
  g_return_if_fail (tree_item_pix != NULL);
  g_return_if_fail (GTK_IS_TREE_ITEM_PIX (tree_item_pix));

  tree_item_pix->collapsed_pixmap = NULL;
  tree_item_pix->collapsed_bitmap = NULL;
  tree_item_pix->expanded_pixmap = NULL;
  tree_item_pix->expanded_bitmap = NULL;
  tree_item_pix->label_position = GTK_TREE_ITEM_PIX_LABEL_LAST;

  gtk_signal_connect (GTK_OBJECT (tree_item_pix), "collapse",
                      GTK_SIGNAL_FUNC (tree_item_pix_collapse), NULL);
  gtk_signal_connect (GTK_OBJECT (tree_item_pix), "expand",
                      GTK_SIGNAL_FUNC (tree_item_pix_expand), NULL);
}

GtkWidget*
gtk_tree_item_pix_new (gchar *label,
                       gchar **collapsed_xpm,
                       gchar **expanded_xpm,
                       guint location)
{
  GtkWidget *tree_item_pix;
  GtkWidget *label_widget;
  GtkWidget *table_widget;
  GtkStyle *style;

  /* Create the object and initialize the label. */
  tree_item_pix = GTK_WIDGET (gtk_type_new (gtk_tree_item_pix_get_type ())); 
  gtk_widget_show (tree_item_pix);
  style = gtk_widget_get_style (tree_item_pix);
  
  label_widget = gtk_label_new(label);
  gtk_misc_set_alignment (GTK_MISC (label_widget), 0.0, 0.5);
  gtk_widget_show (label_widget);

  /* Create the pixmaps for the tree item. */
    if (tree_item_pix->window == NULL) printf ("YUP\n");
  GTK_TREE_ITEM_PIX (tree_item_pix)->collapsed_pixmap =
//    gdk_pixmap_create_from_xpm_d (tree_item_pix->window,
    gdk_pixmap_create_from_xpm_d (GTK_WIDGET (tree_item_pix)->window,
                                  &GTK_TREE_ITEM_PIX (tree_item_pix)->collapsed_bitmap,
                                  &style->bg[GTK_STATE_NORMAL],
                                  collapsed_xpm);

  GTK_TREE_ITEM_PIX (tree_item_pix)->current_pix_widget =
    gtk_pixmap_new (GTK_TREE_ITEM_PIX (tree_item_pix)->collapsed_pixmap,
                    GTK_TREE_ITEM_PIX (tree_item_pix)->collapsed_bitmap);

  gtk_widget_show (GTK_TREE_ITEM_PIX (tree_item_pix)->current_pix_widget);

  if (expanded_xpm != NULL)
    GTK_TREE_ITEM_PIX (tree_item_pix)->expanded_pixmap =
      gdk_pixmap_create_from_xpm_d (GTK_WIDGET (tree_item_pix)->window,
                                    &GTK_TREE_ITEM_PIX (tree_item_pix)->expanded_bitmap,
                                    NULL,
                                    expanded_xpm);
  else
  {
    GTK_TREE_ITEM_PIX (tree_item_pix)->expanded_pixmap =
      GTK_TREE_ITEM_PIX (tree_item_pix)->collapsed_pixmap;
    GTK_TREE_ITEM_PIX (tree_item_pix)->expanded_bitmap =
      GTK_TREE_ITEM_PIX (tree_item_pix)->collapsed_bitmap;
  }

  /* Add the widgets to the container using a table. */
  table_widget = gtk_table_new (1, 2, FALSE);
  gtk_widget_show (table_widget);

  if (location == GTK_TREE_ITEM_PIX_LABEL_FIRST)
  {  
  gtk_table_attach (GTK_TABLE (table_widget), label_widget, 0, 1, 0, 1,
    GTK_SHRINK, GTK_SHRINK, 0, 0);
  gtk_table_attach (GTK_TABLE (table_widget),
    GTK_TREE_ITEM_PIX (tree_item_pix)->current_pix_widget, 1, 2, 0, 1,
    GTK_SHRINK, GTK_SHRINK, 0, 0);
  }
  else
  {  
  gtk_table_attach (GTK_TABLE (table_widget),
    GTK_TREE_ITEM_PIX (tree_item_pix)->current_pix_widget, 0, 1, 0, 1,
    GTK_SHRINK, GTK_SHRINK, 0, 0);
  gtk_table_attach (GTK_TABLE (table_widget), label_widget, 1, 2, 0, 1, GTK_SHRINK,
                               GTK_SHRINK, 0, 0);
  }

  gtk_container_add (GTK_CONTAINER (tree_item_pix), table_widget);

  return tree_item_pix;
}

void tree_item_pix_collapse (GtkWidget *item, gpointer data)
{
  gtk_pixmap_set (GTK_PIXMAP (GTK_TREE_ITEM_PIX (item)->current_pix_widget), GTK_TREE_ITEM_PIX (item)->collapsed_pixmap, GTK_TREE_ITEM_PIX (item)->collapsed_bitmap); 
}

void tree_item_pix_expand (GtkWidget *item, gpointer data)
{
  gtk_pixmap_set (GTK_PIXMAP (GTK_TREE_ITEM_PIX (item)->current_pix_widget), GTK_TREE_ITEM_PIX (item)->expanded_pixmap, GTK_TREE_ITEM_PIX (item)->expanded_bitmap); 
}

//void tree_item_pix_selection_changed (GtkWidget *item, gpointer data)
